#!/usr/bin/env python
import sys
sys.path.append(sys.path[0]+"/src") #needed so code can be imported
import gameIO
import globs

class Field:
	def __init__(self, (x,y), walls, dots, blacks, whites):
		#walls and dots are arrays of 3d coordinates
		#stairs is a dictionary with 3d coordinates as keys
		#and 1 or -1 as entries for up and down
		self.dim = (x,y)
		self.walls = walls
		self.dots = dots
		self.blacks = blacks
		self.whites = whites

	def update(self, gameState):
		if gameState.hood.pos in self.dots:
			self.dots.remove(gameState.hood.pos)

	def isEmpty(self):
		return len(self.dots) == 0

class Hood:
	def __init__(self, (x,y)):
		self.pos = (x,y)
		self.facing = (0,1)
		self.dirDict = {globs.KEYCODE["left"]:(-1,0), globs.KEYCODE["right"]:(1,0),\
				globs.KEYCODE["up"]:(0,-1), globs.KEYCODE["down"]:(0,1)}

	def update(self, gameState):
		key = gameIO.getInput(gameState)
		while 1:
			if key == globs.KEYCODE["7"]:
				gameState.field.dots = []
				return
			elif key in self.dirDict:
				newPos = (self.pos[0]+self.dirDict[key][0], self.pos[1]+self.dirDict[key][1])
				if newPos not in gameState.field.walls:
					self.pos = newPos
					self.facing = self.dirDict[key]
					return
			key = gameIO.getInput(gameState)

class Wolf:
	def __init__(self, (x,y)):
		self.pos = (x,y)
		self.prevPos = None
		self.facing = (0,1)
		self.dirList = [(1,0), (-1,0), (0,1), (0,-1)]
		self.LOS = []

	def __getLOS__(self, gameState):
		pos = self.pos
		self.LOS = [(pos[0], pos[1])]
		dx = -1
		while (pos[0]+dx, pos[1]) not in gameState.field.walls:
			self.LOS.append((pos[0]+dx, pos[1]))
			dx -= 1
		dx = 1
		while (pos[0]+dx, pos[1]) not in gameState.field.walls:
			self.LOS.append((pos[0]+dx, pos[1]))
			dx += 1 
		dy = 1
		while (pos[0], pos[1]+dy) not in gameState.field.walls:
			self.LOS.append((pos[0], pos[1]+dy))
			dy += 1
		dy = -1
		while (pos[0], pos[1]+dy) not in gameState.field.walls:
			self.LOS.append((pos[0], pos[1]+dy))
			dy -= 1


	def __neighbours__(self, pos, prevPos, gameState):
		neighbours = []
		for d in self.dirList:
			n = (pos[0]+d[0], pos[1]+d[1])
			if n not in gameState.field.walls:
				neighbours.append(n)
		if pos in gameState.field.blacks: #must turn
			n = (2*pos[0]-prevPos[0], 2*pos[1]-prevPos[1])
			if n in neighbours: neighbours.remove(n)
			if prevPos in neighbours: neighbours.remove(prevPos)
		if pos in gameState.field.whites: #must go straight
			n = (2*pos[0]-prevPos[0], 2*pos[1]-prevPos[1])
			if n not in gameState.field.walls:
				neighbours = [n]
			else:
				neighbours = []

		return neighbours

	def update(self, gameState):
		#an implementation of Dijkstra's algorithm
		blacks = gameState.field.blacks
		whites = gameState.field.whites
		
		known = {(self.prevPos, self.pos): (0, (None,None))} #keys are (prevPos, Pos), values are (distances, prevKey)
		added = (self.prevPos, self.pos)
		visited = {}

		while 1:
			for neighbour in self.__neighbours__(added[1], added[0], gameState):
				distance = known[added][0]+1
				n = (added[1], neighbour)
				if (n not in visited and n not in known) or (n in visited and distance < visited[n][0]):
					visited[n] = (distance, added)

			best, bestV = 10000, None
			for v in visited:
				if visited[v][0] < best:
					best, bestV = visited[v][0], v
			added = bestV
			known[added] = visited.pop(added)

			"""
			#check if hood in LOS:
			tmpPos = self.pos
			self.pos = added[1]
			if self.see(gameState):
				self.pos = tmpPos
				break
			self.pos = tmpPos
			"""
			if added[1] == gameState.hood.pos: break

		toGo = added

		while known[toGo][1] != (self.prevPos, self.pos):
			toGo = known[toGo][1]

		self.prevPos = self.pos
		self.pos = toGo[1]
		self.facing = (self.pos[0] - self.prevPos[0], self.pos[1] - self.prevPos[1])
		#self.__getLOS__(gameState)

	def see(self, gameState):
		hoodX, hoodY = gameState.hood.pos[0], gameState.hood.pos[1]
		self.__getLOS__(gameState)
		if (hoodX, hoodY) in self.LOS:
			return 1
		else:
			return 0
		"""
		x, y = self.pos[0], self.pos[1]
		if hoodX == x:
			dy = 1 if hoodY > y else -1
			for a in range(abs(hoodY-y)+1):
				if (x, y+dy*a) in gameState.field.walls:
					return 0
			return 1
		if hoodY == y:
			dx = 1 if hoodX > x else -1
			for a in range(abs(hoodX-x)+1):
				if (x+dx*a, y) in gameState.field.walls:
					return 0
			return 1
		else:
			return 0
		"""

class GameState:
	def __init__(self, hood, wolves, field, meta):
		self.hood = hood
		self.wolves = wolves
		self.field = field
		self.meta = meta

	def evalState(self):
		if self.field.isEmpty():
			return globs.STATE["win"] # player
		for wolf in self.wolves:
			if wolf.see(self):
				return globs.STATE["lose"]
		return globs.STATE["continue"]

	def update(self):
		self.hood.update(self)
		for wolf in self.wolves:
			wolf.update(self)
		self.field.update(self)

def loadLevel(levelStr):
	lv = __import__(levelStr)
	hood = Hood(lv.hoodPos)
	wolves = [Wolf(wPos) for wPos in lv.wolves]
	dim = lv.fieldDim
	walls = []
	dots = []
	blacks = []
	whites = []
	x,y = 0,0
	for line in lv.field.split("\n"):
		for c in line:
			if c == "#": walls.append((x,y))
			elif c == ".": dots.append((x,y))
			x,y = x+1,y
		x,y = 0,y+1

	x,y = 0,0
	for line in lv.bw.split("\n"):
		for c in line:
			if c == "o": whites.append((x,y))
			elif c == "x": blacks.append((x,y))
			x,y = x+1,y
		x,y = 0,y+1
	field = Field(dim, walls, dots, blacks, whites)
	gameState = GameState(hood, wolves, field, lv.meta)
	for wolf in gameState.wolves:
		wolf.__getLOS__(gameState)
	return gameState

def playLevel(level):
	state = globs.STATE["continue"]
	gameState = loadLevel(level)

	gameIO.drawOutput(gameState)

	while state == globs.STATE["continue"]:
		gameState.update()
		state = gameState.evalState()
		gameIO.drawOutput(gameState)
	gameIO.wait(0.5)
	return state

if __name__ == "__main__":
	sys.path.append(sys.path[0]+"/levels") #needed so levels can be imported
	gameIO.initIO()

	gameIO.drawMenu()

	while playLevel("level1") != globs.STATE["win"]: pass
	while playLevel("level2") != globs.STATE["win"]: pass
	while playLevel("level3") != globs.STATE["win"]: pass
	while playLevel("level4") != globs.STATE["win"]: pass
	while playLevel("level5") != globs.STATE["win"]: pass
	while playLevel("level6") != globs.STATE["win"]: pass
	while playLevel("level7") != globs.STATE["win"]: pass
	while playLevel("level8") != globs.STATE["win"]: pass
	while playLevel("level9") != globs.STATE["win"]: pass
	while playLevel("level10") != globs.STATE["win"]: pass

	gameIO.endIO()
