import pygame, sys, random
from pygame.locals import *
import globs

RESOLUTION = (640, 480)
GRID_SIZE = 16
FG_ALPHA_MIN = 100
FG_ALPHA = 255
GUIDE_ON = 1

def loadImage(imgStr, colorKey=1):
	surf = pygame.image.load(imgStr)
	if colorKey:
		surf.set_colorkey(surf.get_at((0,0)))
	surf.convert_alpha()
	return surf

def drawIsometric(image, (x,y), alpha=255):
	drawX = RESOLUTION[0]/2-GRID_SIZE + GRID_SIZE*(x-y)
	drawY = RESOLUTION[1]/8 + GRID_SIZE/2*(x+y)
	image.set_alpha(alpha)
	screen.blit(image, (drawX, drawY))

def initIO():
	global screen, clock, IMAGE, FONT, MUSIC
	pygame.init()
	screen = pygame.display.set_mode(RESOLUTION)
	clock = pygame.time.Clock()

	IMAGE = {}
	IMAGE["tile1"] = loadImage("art/tile1.bmp")
	IMAGE["tileBlack"] = loadImage("art/tile_black.bmp")
	IMAGE["tileWhite"] = loadImage("art/tile_white.bmp")
	IMAGE["tileDanger"] = loadImage("art/tile_danger.bmp")
	IMAGE["apple"] = loadImage("art/apple.bmp")
	IMAGE["hoodfl"] = loadImage("art/hoodfl.bmp")
	IMAGE["hoodfr"] = loadImage("art/hoodfr.bmp")
	IMAGE["hoodbl"] = loadImage("art/hoodbl.bmp")
	IMAGE["hoodbr"] = loadImage("art/hoodbr.bmp")
	IMAGE["wolfbl"] = loadImage("art/wolfbl.bmp")
	IMAGE["wolfbr"] = loadImage("art/wolfbr.bmp")
	IMAGE["wolffl"] = loadImage("art/wolffl.bmp")
	IMAGE["wolffr"] = loadImage("art/wolffr.bmp")
	IMAGE["bar"] = loadImage("art/bar.bmp", colorKey=0)
	IMAGE["keyGuide"] = loadImage("art/key_guides.bmp")
	IMAGE["title"] = loadImage("art/title.bmp")
	FONT = pygame.font.Font("art/graph_35_pix.ttf", 8)

	pygame.display.set_icon(IMAGE["apple"])
	pygame.display.set_caption("Little Red Riding hood")

def endIO():
	pygame.quit()
	sys.exit()

def drawMeta(gameState):
	meta = gameState.meta 
	title = FONT.render(str(meta["levelNumber"]) + ".  " + str(meta["title"]), 0, (255,255,255))
	tutorial = FONT.render(str(meta["tutMessage"]), 0, (255,255,255))
	titleW, titleH = FONT.size(str(meta["levelNumber"]) + ".  " + str(meta["title"]))
	keyText = FONT.render("Movement Guide", 0, (255,255,255))

	screen.blit(IMAGE["bar"], (0,0))
	screen.blit(IMAGE["bar"], (0,464))
	screen.blit(title, (640 - 25 - titleW,5))
	screen.blit(tutorial, (25 ,469))
	if GUIDE_ON:
		screen.blit(IMAGE["keyGuide"], (530,48))
		screen.blit(keyText, (528, 32))

	pygame.display.set_caption("Little Red Riding hood   [ "+str(meta["levelNumber"])+". "+str(meta["title"])+" ]")

def drawOutput(gameState):
	HOOD_SPRITE = {(-1,0):IMAGE["hoodbl"], (1,0):IMAGE["hoodfr"],\
		       (0,1):IMAGE["hoodfl"], (0,-1):IMAGE["hoodbr"]}
	WOLF_SPRITE = {(-1,0):IMAGE["wolfbl"], (1,0):IMAGE["wolffr"],\
		       (0,1):IMAGE["wolffl"], (0,-1):IMAGE["wolfbr"]}

	field = gameState.field
	walls = gameState.field.walls
	dots = gameState.field.dots
	blacks = gameState.field.blacks
	whites = gameState.field.whites
	wolves = gameState.wolves
	hood = gameState.hood
	bgCol = (0, 0, 5)

	screen.fill(bgCol)
	coords = [(x,y) for x in range(field.dim[0]) for y in range(field.dim[1])]
	for (x,y) in coords:
		pos = (x,y)
		if pos not in walls:
			drawIsometric(IMAGE["tile1"], pos)
		for wolf in wolves:
			if pos in wolf.LOS:
				drawIsometric(IMAGE["tileDanger"], pos, 200)
		if pos in blacks:
			drawIsometric(IMAGE["tileBlack"], pos, 220)
		if pos in whites:
			drawIsometric(IMAGE["tileWhite"], pos, 180)
		if pos in dots:
			drawIsometric(IMAGE["apple"], pos, FG_ALPHA)
		for wolf in wolves:
			if pos == wolf.pos:
				drawIsometric(WOLF_SPRITE[wolf.facing], pos, FG_ALPHA)
		if pos == hood.pos:
			drawIsometric(HOOD_SPRITE[hood.facing], pos, FG_ALPHA)

	drawMeta(gameState)

	pygame.display.update()
	clock.tick(30)

def getInput(gameState):
	global FG_ALPHA, GUIDE_ON
	for event in pygame.event.get():
		if event.type == KEYDOWN:
			if event.key == K_UP:
				return globs.KEYCODE["up"]
			elif event.key == K_DOWN:
				return globs.KEYCODE["down"]
			elif event.key == K_LEFT:
				return globs.KEYCODE["left"]
			elif event.key == K_RIGHT:
				return globs.KEYCODE["right"]
			elif event.key == K_7:
				return globs.KEYCODE["7"]
			elif event.key == K_SPACE:
				FG_ALPHA = FG_ALPHA_MIN 
				drawOutput(gameState)
			elif event.key == K_t:
				GUIDE_ON ^= 1
				drawOutput(gameState)
		elif event.type == KEYUP:
			if event.key == K_SPACE:
				FG_ALPHA = 255 
				drawOutput(gameState)
		elif event.type == QUIT:
			endIO()
				
	return None

def wait(s):
	pygame.time.wait(int(1000*s))

def drawMenu():
	msg1 = FONT.render("Help can be found on the bottom of the screen!", 0, (255,255,255))
	size1 = FONT.size("Help can be found on the bottom of the screen!")
	msg2 = FONT.render("Pyweek 15 entry by Kcaze", 0, (255,255,255))
	size2 = FONT.size("Pyweek 15 entry by Kcaze")

	screen.fill((0,0,0))
	screen.blit(IMAGE["title"], (320-135, 180-159/2))
	screen.blit(msg1, (320-size1[0]/2, 180+159/2+17))
	screen.blit(msg2, (640-size2[0]-16, 464))

	pygame.display.update()
	wait(3)
